/// <reference types="node" />
import { EventEmitter } from 'events';
export declare namespace System {
    interface Options {
        name?: string;
        exitOnError?: boolean;
        terminationTimeout?: number;
    }
    interface ResourceDescriptor {
        [resourceName: string]: any;
    }
    interface Dependency {
        component: string;
        as?: string;
        source?: string;
    }
    type ComponentStartFunction = (resources?: ResourceDescriptor, restart?: ComponentRestartFunction, stop?: ComponentStopFunction) => Promise<any>;
    type ComponentRestartFunction = () => void;
    type ComponentStopFunction = () => Promise<void>;
    type RestartFunction = (component: System.Component) => void;
    type StopFunction = (component: System.Component, error?: Error) => void;
    interface Component {
        name?: string;
        start: ComponentStartFunction;
        stop?: ComponentStopFunction;
        dependencies?: Dependency[];
        mandatory?: boolean;
    }
    type ComponentDelta = {
        [p in keyof Component]?: Component[p];
    };
}
export declare class System extends EventEmitter {
    private components;
    private options;
    private running;
    constructor(options?: System.Options);
    private setOptions({name, exitOnError, terminationTimeout});
    private listenSignals();
    private updateLast(delta);
    dependsOn(...deps: (string | System.Dependency)[]): this;
    ignorable(): this;
    add(name: string, component: System.Component): this;
    start(initResources?: System.ResourceDescriptor, restart?: System.RestartFunction, interrupt?: System.StopFunction): Promise<System.ResourceDescriptor>;
    stop(): Promise<void>;
    restart(component?: System.Component): Promise<System.ResourceDescriptor>;
    group(): System.Component;
    logAllEvents(): this;
    private handleInterruption(component, error?);
    private handleProcessMessage(msg);
    private handleSignal(signal);
    private handleUncaughtException(error);
    private handleUnhandledRejection(error);
    private handleTerminationTimeout();
    private handleError(error?);
    private handleExit(code);
    private gracefulTerminate(error?);
    private exit(code);
}
export default System;
export declare function createDependency(dep: string | System.Dependency): System.Dependency;
export declare function mapResources(allResources: System.ResourceDescriptor, dependencies: System.Dependency[]): System.ResourceDescriptor;
export declare function filterResources(initResources: System.ResourceDescriptor, subSystemResources: System.ResourceDescriptor): System.ResourceDescriptor;
export declare function start([first, ...others]: System.Component[], resources: System.ResourceDescriptor, restart: System.RestartFunction, stop: System.StopFunction, onComponentStart: (componentName: string, resources: System.ResourceDescriptor) => void, onComponentStartFailed: (componentName: string, error: Error) => void): Promise<System.ResourceDescriptor>;
export declare function stop([first, ...others]: System.Component[], onComponentStop: (componentName: string) => void, onComponentStopFailed: (componentName: string, error: Error) => void): Promise<void>;
