"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const DependencySorter = require("dependency-sorter");
function assertDependencies(components) {
    components.forEach(component => {
        component.dependencies.forEach(dep => {
            if (!components.find(({ name }) => name === dep.component))
                throw new Error(`${component.name} has ${dep.component} dependency, but it does not exist in the system`);
        });
    });
}
exports.assertDependencies = assertDependencies;
function prepare(components) {
    return components
        .map((component) => (Object.assign({}, component, { depends: component.dependencies.map(dep => dep.component) })));
}
exports.prepare = prepare;
function sort(components) {
    if (Array.isArray(components) && components.length === 1)
        return components;
    assertDependencies(components);
    return new DependencySorter({ idProperty: 'name' })
        .sort(prepare(components))
        .map(component => { delete component.depends; return component; });
}
exports.default = sort;
